/*
 * main.c
 *
 *  Created on: 24-06-2013
 *      Author: Mireosaw Kara
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <util/delay.h>

#include "MYUART/myuart.h"
#include "MK_TERMINAL/mk_term.h"

#include "I2C_TWI/i2c_twi.h"

inline void set_chanel( uint8_t kanal ) {
	ADMUX = (ADMUX & 0b11111000) | kanal;
}

#define MCP_ADR 0b11010000

#define SR 4

typedef struct {
	uint8_t idx;
	uint16_t adc_mid;
	uint16_t ref_adc;
	uint16_t ref_v;
	char v1[3];
	char v2[3];
} TVOL;

TVOL accu1_vol, mcp_vol, nap1, nap2, nap3;


uint16_t asr[SR];
uint16_t mcp_sr[SR];

uint8_t buf[5];

volatile uint16_t wynik;

uint16_t vtab[4];

uint16_t pomiar( uint8_t kanal );
void get_vol( uint16_t adc, TVOL * voltage, uint16_t * sr );

uint16_t min=1024, max, diff;
#define MAX(a,b) ( (a>b)?a:b )
#define MIN(a,b) ( (a<b)?a:b )

#define REF_256 (1<<REFS1)|(1<<REFS0)
#define REF_VCC (1<<REFS0)

#define LED (1<<PB0)
#define LED_ON PORTB |= LED
#define LED_OFF PORTB &= ~LED
#define LED_TOG PORTB ^= LED

int main( void ) {

	DDRB |= LED;

	// inicjalizacja ADC
	ADCSRA |= (1<<ADEN); // wcz ADC
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0); // preskaler
	ADCSRA |= (1<<ADIE); // Free Runing Mode plus Interrupt enable
	ADMUX |= REF_256;	// ustawiamy wewn. r odn. 2,56V

	accu1_vol.ref_adc = 494;
	accu1_vol.ref_v = 600;

	mcp_vol.ref_adc = 15891;
	mcp_vol.ref_v = 600;

	uart_init( __UBRR );

	i2cSetBitrate( 100 );

	tr_cls(0);
	uart_puts( "miernik napicia:" );
	_delay_ms( 1000 );
	tr_cls(1);

	tr_locate(0,3);
	tr_attr(1,YELLOW,BLACK);
	uart_puts( "ATmega8 ");



	uint8_t cfg = 0b10011000;
	MCP_write_buf( MCP_ADR, 1, &cfg);
	uint16_t mcpadc=0;

	set_chanel( PC0 );
	sei();
	ADCSRA |= (1<<ADSC);  // start konwersji

	uint16_t tmp_wynik=0;

	while(1) {

//		wynik = pomiar( PC3 );

		uint8_t sreg = SREG;
		cli();
		tmp_wynik = vtab[3];//wynik;
		SREG = sreg;

		get_vol( tmp_wynik, &accu1_vol, asr );

		max = MAX(max, wynik);
		min = MIN(min, wynik);
		diff = max-min;

		tr_locate(3,3);
		tr_attr(1,MAGENTA,BLACK);
		uart_puts( "Warto ADC: ");
		tr_pen_color(RED);
		uart_putlong( tmp_wynik, 10 );
		uart_puts( "/");
		uart_putlong( accu1_vol.adc_mid, 10 );
		uart_puts( "     min: ");
		uart_putlong( min, 10 );
		uart_puts( "     max: ");
		uart_putlong( max, 10 );
		uart_puts( "     diff: ");
		uart_putlong( diff, 10 );
		uart_puts( "       ");

		tr_locate(5,3);
		tr_attr(1,WHITE,BLACK);
		uart_puts( "napicie: ");
		uart_puts( accu1_vol.v1 );
		uart_puts( ".");
		uart_puts( accu1_vol.v2 );
		uart_puts( " V       ");

		LED_TOG;

		MCP_read_buf(MCP_ADR, 4, buf);

		mcpadc = buf[0]<<8 | buf[1];

		get_vol( mcpadc, &mcp_vol, mcp_sr );

		tr_locate(8,3);
		tr_attr(1,CYAN,BLACK);
		uart_puts( "MCP control: ");
		uart_putlong( buf[3], 16 );
		uart_puts( "     ");
		uart_putlong( buf[2], 16 );
		uart_puts( "     ");
		uart_putlong( buf[1], 16 );
		uart_puts( "     ");
		uart_putlong( buf[0], 16 );
		uart_puts( "     ");

		tr_locate(9,3);
		tr_attr(1,CYAN,BLACK);
		uart_puts( "MCP ADC: ");
		uart_putlong( mcpadc, 10 );
		uart_puts( "/");
		uart_putlong( mcp_vol.adc_mid, 10 );
		uart_puts( "     ");

		tr_locate(11,3);
		tr_attr(1,WHITE,BLACK);
		uart_puts( "MCP3421: ");
		uart_puts( mcp_vol.v1 );
		uart_puts( ".");
		uart_puts( mcp_vol.v2 );
		uart_puts( " V       ");


		tr_locate(13,3);
		tr_attr(1,MAGENTA,BLACK);
		uart_putlong( vtab[0], 10 );
		uart_puts( "       ");

		tr_locate(14,3);
		tr_attr(1,MAGENTA,BLACK);
		uart_putlong( vtab[1], 10 );
		uart_puts( "       ");

		tr_locate(15,3);
		tr_attr(1,MAGENTA,BLACK);
		uart_putlong( vtab[2], 10 );
		uart_puts( "       ");



		_delay_ms( 250 );

	}
}

ISR( ADC_vect ) {
	static uint8_t idx;

	vtab[idx++] = ADCW;
	if( idx>3 ) idx=0;

	ADMUX = (ADMUX & 0b11111000) | idx;

	ADCSRA |= (1<<ADSC);

//	wynik = ADCW;
}

uint16_t pomiar( uint8_t kanal ) {

	ADMUX = (ADMUX & 0b11111000) | kanal;

	ADCSRA |= (1<<ADSC);  // start konwersji

	while( ADCSRA & (1<<ADSC) );

	return ADCW;
}

void get_vol( uint16_t adc, TVOL * voltage, uint16_t * sr ) {

//	static uint16_t sr[SR];
//	static uint8_t idx;

	uint32_t sr1=0;
	uint8_t i;

	sr[ voltage->idx++ & (SR-1) ] = adc;

	for(i=0; i<SR; i++) sr1+=sr[i];
	sr1 /= SR;

	voltage->adc_mid = sr1;

	uint16_t wynik = ( sr1 * (uint32_t)voltage->ref_v ) / voltage->ref_adc;

	div_t divmod = div(wynik, 100);
	itoa( divmod.quot, voltage->v1, 10 );
	itoa( divmod.rem, voltage->v2, 10);
	if( divmod.rem < 10 ) {
		voltage->v2[0]='0';
		voltage->v2[1]=divmod.rem+'0';
	}

	voltage->v1[2]=0;
	voltage->v2[2]=0;
}
